/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Tree-Structured Filter Bank (TSFB) Library
 *
 * $Id$
 */

#ifndef TSFB_H
#define TSFB_H

/******************************************************************************\
* Includes.
\******************************************************************************/

#include "qmfb.h"
#include "seq.h"

/******************************************************************************\
* Constants.
\******************************************************************************/

#define	TSFB_MAXBANDS	(TSFB_MAXDEPTH * 3 + 1)
#define	TSFB_MAXDEPTH	32
#define	TSFB_RITIMODE	QMFB1D_RITIMODE

#define	TSFB_MAXBANDSPERNODE	(QMFB1D_MAXCHANS * QMFB1D_MAXCHANS)

#define	TSFB_PRUNEVERT	0x01
#define	TSFB_PRUNEHORZ	0x02

/******************************************************************************\
* Types.
\******************************************************************************/

typedef struct {

	int xstart;
	int ystart;
	int xend;
	int yend;
	int locxstart;
	int locystart;
	int locxend;
	int locyend;

} tsfbnodeband_t;

typedef struct tsfbnode_s {

	int numhchans;
	int numvchans;
	qmfb1d_t *hqmfb;
	qmfb1d_t *vqmfb;
	int maxchildren;
	int numchildren;
	struct tsfbnode_s *children[TSFB_MAXBANDSPERNODE];
	struct tsfbnode_s *parent;

} tsfbnode_t;

typedef struct {
	tsfbnode_t *root;
} tsfb_t;

typedef struct {
	int xstart;
	int ystart;
	int xend;
	int yend;
	int orient;
	int locxstart;
	int locystart;
	int locxend;
	int locyend;
	fix_t synenergywt;
} tsfb_band_t;

/******************************************************************************\
* Functions.
\******************************************************************************/

tsfb_t *tsfb_wavelet(qmfb1d_t *hqmfb, qmfb1d_t *vqmfb, int numdlvls);

void tsfb_destroy(tsfb_t *tsfb);

void tsfb_analyze(tsfb_t *tsfb, int flags, seq2d_t *x);

void tsfb_synthesize(tsfb_t *tsfb, int flags, seq2d_t *x);

int tsfb_getbands(tsfb_t *tsfb, uint_fast32_t xstart, uint_fast32_t ystart,
  uint_fast32_t xend, uint_fast32_t yend, tsfb_band_t *bands);

void tsfb_prune(tsfb_t *tsfb, int flags, int maxlvls, seq2d_t *x);

int tsfb_getmaxlvls(tsfb_t *tsfb);

#endif
