/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Portable Pixmap/Graymap Format Support
 *
 * $Id$
 */

/******************************************************************************\
* Includes.
\******************************************************************************/

#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <assert.h>

#include "mathx.h"
#include "memalloc.h"
#include "pnm.h"

/******************************************************************************\
* Miscellaneous utilities.
\******************************************************************************/

/* Determine the PNM type (i.e., PGM or PPM) from the magic number. */
int pnm_type(uint_fast16_t magic)
{
	int type;
	switch (magic) {
	case PNM_MAGIC_TXTPPM:
	case PNM_MAGIC_BINPPM:
		type = PNM_TYPE_PPM;
		break;
	case PNM_MAGIC_TXTPGM:
	case PNM_MAGIC_BINPGM:
		type = PNM_TYPE_PGM;
		break;
	default:
		/* This should not happen. */
		abort();
		break;
	}
	return type;
}

/* Determine the PNM format (i.e., text or binary) from the magic number. */
int pnm_fmt(uint_fast16_t magic)
{
	int fmt;
	switch (magic) {
	case PNM_MAGIC_TXTPPM:
	case PNM_MAGIC_TXTPGM:
		fmt = PNM_FMT_TXT;
		break;
	case PNM_MAGIC_BINPPM:
	case PNM_MAGIC_BINPGM:
		fmt = PNM_FMT_BIN;
		break;
	default:
		/* This should not happen. */
		abort();
		break;
	}
	return fmt;
}

/* Determine the depth (i.e., precision) from the maximum value. */
int pnm_maxvaltodepth(uint_fast32_t maxval)
{
	return ceillog2(maxval + 1);
}
